//------------------------------------------------------------------
/*

SPI.c

Routines dealing with communications between the Propeller processor and the DS00VQ100 chip.

*/
//------------------------------------------------------------------
// Constants and global variables
#include <constants.h>
#include <externals.h>
// C libraries
#include <propeller.h>
#include <stdio.h>
#include "propeller-icc.h"

//------------------------------------------------------------------
// Protoypes of external routines
// NONE
//------------------------------------------------------------------ START
char SPI_Byte(char send_char)
/*
    Function to transmit and receive bytes over the SPI bus.
    Transmit and receive occur simultaneously.
    Chip select is handled elsewhere.
*/
    {
    int get_char = 0;
    int out_mask = 0x80;                                    // Used as a bit pointer
    int i = 0;
    
    for(i=0; i<=7; i++)                                        // Send and receive 8 bits
        {
        get_char = get_char << 1;                        // Rotate input buffer ready to receive a bit
        
        if((send_char & out_mask) != 0)            // Check bit on send buffer
            SET(OUTA, MOSI);                                    // Output a 1
        else
            CLR(OUTA, MOSI);                                    // Or a 0

        SET(OUTA, SPI_CLOCK);                                // Provide a rising clock edge
        if((INA & IN_MASK) != 0)                        // If there is an incoming 1 then
            get_char++;                                                // Add it to the LS bit of the receive buffer
        CLR(OUTA, SPI_CLOCK);                                // Provide a falling clock edge
                
        out_mask = out_mask >> 1;                        // Rotate the outpur buffer ready to send the next bit
        }
    return get_char;                                            // When finished, return the input buffer
    }    
//------------------------------------------------------------------
int SPI (void)
/*
    Checks for new data ready signal from DS00VQ100 chip OR a request for an asynchronous data load.
    It then executes a 14 byte transfer on the SPI bus.
    
    Converts 8 bit MSB/LSB data into 16 bit results where necessary.
    Updates other parameters for later processing.

    Inputs:        None
     Returns:     int NOT_READY if no new values are ready.
                        int READY if new values are available for further processing.
*/

    {
    int j, ZF, ZR, SF, SR;
                                                                                                                                    // Check for new data ready signal or asynchronous transfer request
    if(((INA & DATA_READY_MASK) == READY) | (int_settings[FOOTER] == ASYNC_FOOTER))    
    {
    int_settings[DATA_INDEX]++;                                                                        // Update index associated with the next result 
        int_settings[DATA_INDEX] = int_settings[DATA_INDEX] & 0x03;        // Limit index to a count range of 0 - 3
        SET(OUTA, CS);                                                                                                // Select the DS00VQ100 chip
        msleep(1);                                                                                                        // Provide a short pause before starting SPI transfer        
        for(j=0; j<14; j++)                                                                                        // Do a data transfer on the SPI bus
            spi_read[j] = SPI_Byte(int_settings[j]);
        msleep(1);                                                                                                         // Provide a short pause before ending SPI transfer
        CLR(OUTA, CS);                                                                                                 // Deselect the DS00VQ100 chip
        int_settings[FOOTER] = SYNC_FOOTER;                                                        // Always return to synchronous SPI write for next data transfer
        
        switch(int_settings[MEASURING_RANGE])                                                    // Keep the measuring range updated for later calculations
            {
            case 1:
                range = RANGE1;
            break;
            case 2:
                range = RANGE2;
            break;
            case 3:
                range = RANGE3;
            break;
            case 4:
                range = RANGE4;
            break;        
            default:
                range = RANGE0;
            break;
            }
                                                                                                    // Update time expansion direction flag for later use
        if ((spi_read[STATUS] & TIME_DIR_MASK) == FORWARD)            
            direction = FORWARD;                                                
        else
            direction = REVERSE;
                                                                                                    // Convert 8 bit values to 16 bit values where neccesary
        spi_read[NOISE] = spi_read[NOISE_MSB] * 256 + spi_read[NOISE_LSB];
        spi_read[SPAN] = spi_read[SPAN_MSB] * 256 + spi_read[SPAN_LSB];
        ZF = spi_read[ZERO_FRONT_MSB] * 256 + spi_read[ZERO_FRONT_LSB];
        ZR = spi_read[ZERO_REAR_MSB] * 256 + spi_read[ZERO_REAR_LSB];
        SF = spi_read[SIGNAL_FRONT_MSB] * 256 + spi_read[SIGNAL_FRONT_LSB];
        SR = spi_read[SIGNAL_REAR_MSB] * 256 + spi_read[SIGNAL_REAR_LSB];

                                                                                                    // Some 8 bit to 16 bit conversions depend on the direction of time expansion
        if (direction == FORWARD)                                            // Forward time expansion
            {
            spi_read[ZERO_FRONT] = ZF;
            spi_read[ZERO_REAR] = ZR;
            spi_read[SIGNAL_FRONT] = SF;
            spi_read[SIGNAL_REAR] = SR;
            }
        else                                                                                    // Reverse time expansion
            {
            spi_read[ZERO_FRONT] = spi_read[SPAN] - ZR;
            spi_read[ZERO_REAR] = spi_read[SPAN] - ZF;
            spi_read[SIGNAL_FRONT] = spi_read[SPAN] - SR;
            spi_read[SIGNAL_REAR] = spi_read[SPAN] - SF;
            }
            
                                                                                                    // Update signal valid flag for later use
        if (((spi_read[STATUS] & SIGNAL_VALID_MASK) == 0) | (spi_read[SIGNAL_FRONT] == 0))
            valid_signal = FALSE;    
        else
            valid_signal = TRUE;
                    
        if(start_flag >0)                                                            // Count down to wait for a good result after power up
            start_flag--;
        if(start_flag > 1)                                                        // Start using results when start flag reaches 1
            return (NOT_READY);
        return (READY);                                                             // Set the data ready flag when there is new data
    } 
    else    
        return (NOT_READY);                                                        // Clear the data ready flag if there is no new data
    }
/*
-----------------------------------------------------------------------------------------------------------
This work is licensed under the Creative Commons Attribution 3.0 Unported License.
To view a copy of this license, visit http://creativecommons.org/licenses/by/3.0/
or send a letter to Creative Commons, 444 Castro Street, Suite 900, Mountain View, California, 94041, USA.
-----------------------------------------------------------------------------------------------------------
*/    
//------------------------------------------------------------------ END