//------------------------------------------------------------------
/*

Utilities.c
    
Miscellaneous functions used by other routines.
    
*/
//------------------------------------------------------------------
// Constants and global variables
#include <constants.h>
#include <externals.h>
// C libraries
#include <propeller.h>
#include <stdio.h>
#include <math.h>
#include <float.h>
#include <ctype.h>
#include <string.h>
#include "propeller-icc.h"

//------------------------------------------------------------------
// Protoypes of external routines
int EEPROM_read(int address);                                                        // EEPROM.c
int Get_EEPROM_Int(int eeaddress);                                            // EEPROM.c
float Get_EEPROM_Float(int eeaddress);                                    // EEPROM.c
void Save_EEString(char *str, int eeaddress, int len);     // EEPROM.c
char Echo_Key(void);                                                                        // Serial.c
int SPI(void);                                                                                    // SPI.c
//------------------------------------------------------------------ START
void get_rx_flag()
/*
    Creates a flag for incoming USB characters.
    Runs on a separate cog.
    Replaces the hardware "serial data ready" flag on other processors.
*/

    {
    rx_flag = -1;                                                    // Initialise the flag to clear
    while(1)                                                            // Continually check for a character on the USB port
        {
        while((INA & USB_READY_MASK) == 0);    // Check for USB connection and wait if it's not connected
        global_key = getchar();                            // Save the incoming char in a global memory location
        rx_flag = 0;                                                // Set the serial data ready flag
        while(rx_flag == 0);                                // Wait until an external routine has read the data and cleared the flag
        }
    }
//------------------------------------------------------------------
void Flash(int pin)
/*
    Toggles a port pin to flash an LED or sound the buzzer.
*/

    {
    int j;
    
    for (j = 0; j <= 4; j++)        // Flash 5 times
        {
        SET(OUTA,pin);                        // Output high
        msleep(20);                                // Wait
        CLR(OUTA,pin);                        // Output low
        msleep(20);                                // Wait
        }
    }    
//------------------------------------------------------------------
void Show_Strength(void)
/*
    Displays the signal strength on a 0 - 9 scale using the LEDs.
*/

    {
    int j, str;
    
    str = (int)(signal_width_in);                        // Convert the signal width in meters to an integer
                                                                                    // Check for limits
    if(valid_signal == FALSE)                                // If there is no signal use 0
        str = 0;
    if(str > 9)                                                            // If strength > 9 then use 9
        str = 9;
        
    if(int_settings[LASER_VOLTAGE] == 0)        // Turn off display if laser is off
     str = 10;
     
    for (j = 0; j <= 9; j++)                                // Set and clear all LEDs
        {
        if(j == str)                                                    // Turn on the LED indicating signal strength
            SET(OUTA,j);
        else
            CLR(OUTA,j);                                                // Turn off all the others
        }
    }
//------------------------------------------------------------------
/*
    Routine to convert integer values from the DS00VQ100 chip into a floating point distances.
    Applies corrections for signal strength, zero delay and user zero offset.
    Performs averaging if it is turned on.
    Updates the signal strength LEDs.
*/

float To_Dist (int value)                                                    // Useful function used in this routine only
    {
    return((float)(value) * resolution);
    }

void Convert_to_Distance(void)                                        // Main routine starts here
    {    
    int j;
    
    if(spi_read[SPAN] == 0)                                                    // Prevent divide-by-zero by forcing a resolution of 1 if there is no span result
        resolution = 1.0;
    else
        resolution = range / (float)(spi_read[SPAN]);    // Otherwise calculate the resolution in meters
    
                                                                                                    // Calculate the minimum valid signal width to compensate for changes in the span
    int_settings[SIGNAL_WIDTH] = (int)((float)min_width / (resolution * 100));
    int_settings[ZERO_WIDTH] = int_settings[SIGNAL_WIDTH];

    if(valid_signal == TRUE)
        {    
        zero_front = To_Dist(spi_read[ZERO_FRONT]);            // Convert zero leading edge from counts into meters
        signal_front = To_Dist(spi_read[SIGNAL_FRONT]);    // Convert return signal leading edge from counts into meters
                                                                                                        // Calculate the width of the return signal
        signal_width_in = To_Dist(spi_read[SIGNAL_REAR]) - signal_front;
        signal_width_out = signal_width_in;
    
                                                                                                        // Apply various corrections to the return signal    
        j = (int)(signal_width_in);                                            // Firstly, compensate for signal width/strength effects
        if(j > 15)                                                                            // If the signal is more then 15m wide then limit it to 15m
            j = 15;
                                                                                                        // Use linear interpolation to find a correction for the current width
        slope = slopes[j];                                                            // Select a slope based on the integer width
        intercept = intercepts[j];                                            // Select an intercept based on the integer width
        correction = slope * signal_width_in + intercept;    // Calculate correction using y = mx + c

                                                                                                        // Apply all the corrections to calculate the correct distance
        distance = signal_front - zero_front - correction + ((float)int_settings[ZERO_OFFSET])/100;
        //printf("DI:%.2f  CO:%.2f  ", distance, correction);

                                                                                                        // If the averaging flag is set then calculate the rolling averages
                                                                                                        // of the distance and the signal width
        if(int_settings[AVERAGE_FLAG] != 0)                                                        
            {                                                                                            // Update the averaging buffers
            if(start_flag == 0)
                {
                distance_averages[average_pointer] = distance;
                signal_width_averages[average_pointer] = signal_width_out;    
                }
            else
                {                                                                                        // On power up, load all buffers with the first good result
                for(j = 0; j < 16; j++)
                    {
                    distance_averages[j] = distance;
                    signal_width_averages[j] = signal_width_out;
                    }
                }                
            
            average_pointer++;                                                        // Point to the next buffer locations ready for the next result        
            if(average_pointer > 15)                                            // Collect 16 answers only in the buffers
                average_pointer = 0;
    
            distance = 0.0;                                                                // Average the values in the averaging buffers
            signal_width_out = 0.0;
            for(j = 0; j < 16; j++)
                {
                distance = distance + distance_averages[j];
                signal_width_out = signal_width_out + signal_width_averages[j];
                }
            distance = distance / 16.0;
            signal_width_out = signal_width_out / 16.0;
            }
        }
    else
        {
        signal_front = 0.0;
        signal_width_in = 0.0;
        signal_width_out = 0.0;
        distance = 0.0;
        }        
    Show_Strength();                                                                // Display the signal strength on the LEDs
    }

//------------------------------------------------------------------
/*
 Function to blink the LEDs and sound the buzzer five times.
 Only used once on power up.
*/

void BlinkLED(void)
    {
    int i, k;

    for(k = 0; k<=4; k++)
      {
      for (i = LED1; i <= BUZZER; i++)
            Flash(i);
      for (i = BUZZER; i >= LED1; i--)
            Flash(i);
        }
    }
//------------------------------------------------------------------
/*
    Function to convert the resolution from meters into centimeters for display purposes.
    This makes it easier to read the resolution on the terminal screen.    
*/

float Rescm(void)
    {
    float r;
    
    r = resolution * 100.0;                // Convert to centimeters
    if(direction == REVERSE)            // Use -ve sign to show reverse time expansion
        return(0.0 - r);
    return(r);
    }
//------------------------------------------------------------------
/*
    System hardware initialisation.
    Called once by main() on power up.
*/

void Initialise_Hardware(void)
    {
    static long stack[8];
    int cog_id = 0;
        
    //clkset(0x6E, 10000000);                                                                        // (0x6E, 4915200) = PLL x 8, (0x6F, 4915200) = PLL x 16;
    //clkmode : (BYTE RAM [4])
    //clkfreq : Clock Frequency (RAM LONG[0])
    msleep(1000);                                                                                            // Wait for new clock settings to take effect
    //freopen("SSER:57600,31,30", "r+", stdout); 
    DIRA = DIRA | PORT_DIRECTIONS;      // Set port directions
#if defined(__PROPELLER__) && defined(__GNUC__)
    static _thread_state_t thread_data;
    cog_id = _start_cog_thread(&stack[7], get_rx_flag, NULL, &thread_data);
#else
    cog_id = cognew(get_rx_flag, &stack[7]);                                    // Start comms detection on separate cog
#endif

    SET(OUTA, CSNEEPROM);                                                                            // Deselect the EERPOM on the timer board 
    }
//------------------------------------------------------------------    
/*
    Initialise software parameters.
    Called by main() on power up and by the EEPROM editor when new values are stored. 
*/
    
void Initialise_Parameters(void)
    {
    int j;    
    printf("Initialise_Parameters\n");

    for(j=0; j<22; j++)                                                                            // Fetch EEPROM integer values and save locally
        int_settings[j] = Get_EEPROM_Int(j);
        
    for(j=0; j<16; j++)                                                                            // Fetch the EEPROM float values and save locally
        {
        corrections[j] = Get_EEPROM_Float(CORRECTIONS + j);
        //printf("CO %d:%.2f\r", j, corrections[j]);
        }    
                                                                                                                    // Fetch display mode as an ASCII character
    int_settings[DISPLAY_MODE] = tolower(EEPROM_read(DISPLAY_MODE * 16));    
    serial_number = Get_EEPROM_Int(SERIAL_NUMBER);                    // Fetch the serial number of the LRF for display only
    min_width = int_settings[SIGNAL_WIDTH];                                    // Save for later use
    if(int_settings[CONTROL_RESOLUTION] > 9)                                // Limit the range of control to +9/-9 cm
        int_settings[CONTROL_RESOLUTION] = 9;
    if(int_settings[CONTROL_RESOLUTION] < -9)
        int_settings[CONTROL_RESOLUTION] = -9;
    if(int_settings[CONTROL_RESOLUTION] == 0)
        int_settings[CONTROL_RESOLUTION_FLAG] = 0;    
        
    for(j = 0; j < 15; j++)                                                                    // Pre-calculate the signal width linearisation table for later use
        {
        slopes[j] = corrections[j + 1] - corrections[j];
        intercepts[j] = corrections[j] - slopes[j] * (float)j;
        //printf("%d SL:%.2f IN:%.2f\r", j, slopes[j], intercepts[j]);
        }
    
    int_settings[FOOTER] = ASYNC_FOOTER;                                        // Set the SPI footer for immediate load into the DS00VQ100 chip
    start_flag = RESTART;                                                                        // Notify later routines that this is a cold start situation
    }
//------------------------------------------------------------------    
/*
    Save all software parameters.
    Saves the current values from processor memory to EEPROM. 
*/    

void Save_Memory(void)
    {
    int j;
    char buff[16];    
    
    printf("\r Overwrite EEPROM(Y/N)? ");                        // Check with the user that they want to save all the settings
    if (Echo_Key() == 'y')
        {
        int_settings[SIGNAL_WIDTH] = min_width;                // Revert to value originally loaded
        int_settings[ZERO_WIDTH] = min_width;                    // Revert to value originally loaded
        for(j = 0; j < 20; j++)                                                // Save the settings
            {            
            sprintf(buff,"%d",int_settings[j]);    
            Save_EEString(buff, j, strlen(buff));
            }
        }    
    }
/*
-----------------------------------------------------------------------------------------------------------
This work is licensed under the Creative Commons Attribution 3.0 Unported License.
To view a copy of this license, visit http://creativecommons.org/licenses/by/3.0/
or send a letter to Creative Commons, 444 Castro Street, Suite 900, Mountain View, California, 94041, USA.
-----------------------------------------------------------------------------------------------------------
*/
//------------------------------------------------------------------ END





